<?php

namespace Drupal\azure_oauth_sso\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

class oauthRolesMappingForm extends ConfigFormBase
{

    protected function getEditableConfigNames()
    {
        return ['azure_oauth_sso_roles_mapping.settings'];
    }

    public function getFormId()
    {
        return 'azure_oauth_sso_roles_mapping_form';
    }

    public function buildForm(array $form, FormStateInterface $form_state)
    {
        $config = $this->config('azure_oauth_sso_roles_mapping.settings');

        $roles = user_roles(true);

        $role_options = [];
        foreach ($roles as $role) {
            $role_options[$role->id()] = $role->label();
        }

        $form['default_role'] = [
            '#type' => 'select',
            '#title' => t('Default Role'),
            '#options' => $role_options,
            '#description' => t('Choose a role for the user.'),
            '#required' => true,
            '#default_value' => $config->get('default_role'),
        ];

        $form['role_options'] = [
            '#type' => 'radios',
            '#title' => t('Mapping for'),
            '#id' => 'role-options-mapping',
            '#options' => [
                'ad_group' => t('Microsoft Group'),
                'department_field' => t('Microsoft Department Field'),
            ],
            '#default_value' => $config->get('role_options') ?? null,
            '#description' => t("* If the (AD Group) option is chosen, you must enter the group Opject ID from AD.<br>* If the (Department Field Property) is chosen, you must enter the value of the department field from AD."),
        ];

        $form['custom_attribute_mapping']['custom_attr_table'] = [
            '#type' => 'table',
            '#id' => 'attribute-mapping',
            '#header' => [
                $this->t('Drupal Roles'),
            ],
            '#empty' => $this->t('No custom attributes found'),
            '#states' => [
                'visible' => [
                    ':input[name="enable_role_mapping"]' => ['checked' => true],
                ],
            ],
        ];

        foreach ($role_options as $role_key => $role_label) {
            $form['custom_attribute_mapping']['custom_attr_table'][$role_key] = [
                '#states' => [
                    'visible' => [
                        ':input[name="enable_role_mapping"]' => ['checked' => true],
                    ],
                ],
                'drupal_attribute' => [
                    'data' => [
                        '#type' => 'checkbox',
                        '#title' => $this->t('@label', ['@label' => $role_label]),
                        '#attributes' => [
                            'class' => ['custom-attribute-checkbox'],
                        ],
                    ],
                ],
                'textfield' => [
                    'data' => [
                        '#type' => 'textfield',
                        '#default_value' => $config->get("custom_attr_table.$role_key", ''),
                        '#attributes' => [
                            'disabled' => !$config->get("custom_attr_table.$role_key"),
                        ],
                    ],
                ],
            ];
        }

        $form['custom_attribute_mapping']['custom_attr_table']['#multiple'] = true;
        $form['custom_attribute_mapping']['custom_attr_table']['#empty'] = $this->t('No custom attributes found');

        $form['#attached'] = [
            'library' => [
                'azure_oauth_sso/roles-mapping',
            ],
        ];

        return parent::buildForm($form, $form_state);
    }

    public function submitForm(array &$form, FormStateInterface $form_state)
    {
        parent::submitForm($form, $form_state);

        $config = $this->config('azure_oauth_sso_roles_mapping.settings');
        $config->set('default_role', $form_state->getValue('default_role'));
        $config->set('role_options', $form_state->getValue('role_options'));

        $customAttrTable = $form_state->getValue('custom_attr_table');

        foreach ($customAttrTable as $roleKey => $values) {
            if (!empty($values['drupal_attribute'])) {
                $config->set("custom_attr_table.$roleKey", $values['textfield']['data']);
            }
        }

        $config->save();
    }
}
