<?php

namespace Drupal\azure_oauth_sso\Service;

use Drupal\azure_oauth_sso\BaseOAuth;
use Drupal\user\Entity\User;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\RequestException;

class OAuthTokenService {

  use BaseOAuth;

  public function getToken() {
    $user = User::load(\Drupal::currentUser()->id());
    return $user->get('field_refresh_token')->getValue()[0]['value'];
  }

  public function refreshToken() {
    $client = new Client();

    $user = User::load(\Drupal::currentUser()->id());
    $refreshToken = $user->get('field_refresh_token')->getValue()[0]['value'];

    $res = $client->post('https://login.microsoftonline.com/' . $this->getAdTenant() . '/oauth2/v2.0/token', [
      'form_params' => [
        'client_id'     => $this->getClientId(),
        'refresh_token' => $refreshToken,
        'grant_type'    => 'refresh_token',
        'client_secret' => $this->getClientSecret(),
      ],
    ]);

    $response = json_decode($res->getBody(), TRUE);
    if (isset($response['access_token']) && isset($response['refresh_token'])) {
      $user->set("field_access_token", 'Bearer ' . $response["access_token"]);
      $user->set("field_refresh_token", $response["refresh_token"]);
      $user->save();

      return 'Bearer ' . $response["access_token"];
    }
  }

  public function apiCall($url, $params = [], $method = 'POST') {
    $client = new Client();

    try {
      $response = $client->request($method, $url, $params);

      $contentType = $response->getHeader('Content-Type')[0];
      if (strpos($contentType, 'image') !== FALSE) {
        return $response->getBody()->getContents();
      }

      $responseData = json_decode($response->getBody()->getContents(), TRUE);
      return $responseData;
    }
    catch (RequestException $e) {
      if ($e->hasResponse()) {
        $statusCode = $e->getResponse()->getStatusCode();

        if ($statusCode == 401 || $statusCode == 400) {
          $params['headers']['Authorization'] = $this->refreshToken();
          return $this->apiCall($url, $params, $method);
        }
      }
    }
  }
}
